// TutorialView.cpp : implementation of the CTutorialView class
//

#include "stdafx.h"
#include "Tutorial.h"

#include "TutorialDoc.h"
#include "TutorialView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTutorialView

IMPLEMENT_DYNCREATE(CTutorialView, CView)

BEGIN_MESSAGE_MAP(CTutorialView, CView)
	//{{AFX_MSG_MAP(CTutorialView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTutorialView construction/destruction

CTutorialView::CTutorialView()
{
	// TODO: add construction code here
	m_hGLContext = NULL;
	m_GLPixelIndex = 0;
}

CTutorialView::~CTutorialView()
{
}

BOOL CTutorialView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs
	cs.style |= (WS_CLIPCHILDREN | WS_CLIPSIBLINGS);
	cs.lpszClass = AfxRegisterWndClass(CS_OWNDC);

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CTutorialView drawing

void CTutorialView::OnDraw(CDC* pDC)
{
	CTutorialDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CTutorialView diagnostics

#ifdef _DEBUG
void CTutorialView::AssertValid() const
{
	CView::AssertValid();
}

void CTutorialView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CTutorialDoc* CTutorialView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CTutorialDoc)));
	return (CTutorialDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTutorialView message handlers

BOOL CTutorialView::SetWindowPixelFormat(HDC hDC)
{
	PIXELFORMATDESCRIPTOR pixelDesc;

	pixelDesc.nSize		= sizeof(PIXELFORMATDESCRIPTOR);
	pixelDesc.nVersion	= 1;

	pixelDesc.dwFlags	=	PFD_DRAW_TO_WINDOW | 
							PFD_SUPPORT_OPENGL | 
							PFD_DOUBLEBUFFER |
							PFD_STEREO_DONTCARE;

	pixelDesc.iPixelType		= PFD_TYPE_RGBA;
	pixelDesc.cColorBits		= 32;
	pixelDesc.cRedBits			= 8;
	pixelDesc.cRedShift			= 16;
	pixelDesc.cGreenBits		= 8;
	pixelDesc.cGreenShift		= 8;
	pixelDesc.cBlueBits			= 8;
	pixelDesc.cBlueShift		= 0;
	pixelDesc.cAlphaBits		= 0;
	pixelDesc.cAlphaShift		= 0;
	pixelDesc.cAccumBits		= 64;	
	pixelDesc.cAccumRedBits		= 16;
	pixelDesc.cAccumGreenBits	= 16;
	pixelDesc.cAccumBlueBits	= 16;
	pixelDesc.cAccumAlphaBits	= 0;
	pixelDesc.cDepthBits		= 32;
	pixelDesc.cStencilBits		= 8;
	pixelDesc.cAuxBuffers		= 0;
	pixelDesc.iLayerType		= PFD_MAIN_PLANE;
	pixelDesc.bReserved			= 0;
	pixelDesc.dwLayerMask		= 0;
	pixelDesc.dwVisibleMask		= 0;
	pixelDesc.dwDamageMask		= 0;

	m_GLPixelIndex = ChoosePixelFormat(hDC, &pixelDesc);
	if (m_GLPixelIndex==0)
	{
		m_GLPixelIndex = 1;	// Let's choose a default index.
		if (DescribePixelFormat( hDC, m_GLPixelIndex, sizeof(PIXELFORMATDESCRIPTOR), &pixelDesc) == 0)
		{
			DWORD errorCode = GetLastError();
			CString errorStr;
			errorStr.Format("DescribePixelFormat returned error code %d.", errorCode);
			AfxMessageBox(errorStr);
			return FALSE;
		}
	}

	if (SetPixelFormat(	hDC, m_GLPixelIndex, &pixelDesc) ==FALSE)
	{
		DWORD errorCode = GetLastError();
		CString errorStr;
		errorStr.Format("SetPixelFormat returned error code %d.", errorCode);
		AfxMessageBox(errorStr);
		return FALSE;
	}
	return TRUE;
}

int CTutorialView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// TODO: Add your specialized creation code here
	HWND hWnd = GetSafeHwnd();
	HDC hDC = ::GetDC(hWnd);

	if (SetWindowPixelFormat(hDC)==FALSE)
		return 0;

	if (CreateViewGLContext(hDC)==FALSE)
		return 0;

	return 0;
}
	
BOOL CTutorialView::CreateViewGLContext(HDC hDC)
{
	m_hGLContext = wglCreateContext(hDC);
	if (m_hGLContext == NULL)
	{
		DWORD errorCode = GetLastError();
		CString errorStr;
		errorStr.Format("wglCreateContext returned error code %d.", errorCode);
		AfxMessageBox(errorStr);
		return FALSE;
	}

	if (wglMakeCurrent(hDC, m_hGLContext)==FALSE)
	{
		DWORD errorCode = GetLastError();
		CString errorStr;
		errorStr.Format("wglCreateContext returned error code %d.", errorCode);
		AfxMessageBox(errorStr);
		return FALSE;
	}

	return TRUE;
}

void CTutorialView::OnDestroy() 
{
	if(wglGetCurrentContext()!=NULL) 
	{
		// make the rendering context not current
		wglMakeCurrent(NULL, NULL) ;
	}
	
	if (m_hGLContext!=NULL)
	{
		wglDeleteContext(m_hGLContext);
		m_hGLContext = NULL;
	}

	// Now the associated DC can be released.
	CView::OnDestroy();
	
	// TODO: Add your message handler code here
	
}
